;;; APPLICATION TITLE
AppTitle "FunIconTris"

;;; RANDOM SEED
SeedRnd MilliSecs()

;;; RESIZE FACTOR FOR ALL GRAPHICS
Const RESIZE_FACTOR = 40

;;; SCREEN RESOLUTION: 12 x 15 BRICKS
Graphics 12*RESIZE_FACTOR, 15*RESIZE_FACTOR, 32, 2
SetBuffer BackBuffer()
Global frameTimer = CreateTimer(60)

;;; WALL
Global wall = LoadImage("gfx/wall2.png")
ResizeImage wall,RESIZE_FACTOR,RESIZE_FACTOR

;;; FUN ICONS FOR BRICKS
Const BRICK_AMOUNT = 14
Dim brickIcons(BRICK_AMOUNT)
brickIcons(0) = LoadImage("gfx/ball.png")
brickIcons(1) = LoadImage("gfx/crate.png")
brickIcons(2) = LoadImage("gfx/cheese.png")
brickIcons(3) = LoadImage("gfx/key.png")
brickIcons(4) = LoadImage("gfx/shield.png")
brickIcons(5) = LoadImage("gfx/star.png")
brickIcons(6) = LoadImage("gfx/stop.png")
brickIcons(7) = LoadImage("gfx/doubleaxe.png")
brickIcons(8) = LoadImage("gfx/tree.png")
brickIcons(9) = LoadImage("gfx/tree2.png")
brickIcons(10) = LoadImage("gfx/wheel.png")
brickIcons(11) = LoadImage("gfx/sword.png")
brickIcons(12) = LoadImage("gfx/smiley.png")
brickIcons(13) = LoadImage("gfx/axe.png")
brickIcons(14) = LoadImage("gfx/barrel.png")

Global i
For i=0 To BRICK_AMOUNT
	MaskImage brickIcons(i),255,0,255
	ResizeImage brickIcons(i),RESIZE_FACTOR,RESIZE_FACTOR
Next

;;; CLEAR SCREEN COLOR
ClsColor 220,220,220

;;; FORMS
Dim form.Brick(3)
Const BRICK_START_Y = 14
Global formSort
Global formRotation = 0

;;; BRICK POSITIONS (= BLOCKS)
Dim block(11,14)
For i=0 To 14
	block(0,i) = 1
	block(11,i) = 1
Next
For i=0 To 11
	block(i,0) = 1
Next

;;; FALL SPEED
Const FALL_SPEED_INITIAL = 800
Global fallSpeedDelta = FALL_SPEED_INITIAL

;;; GRAVITY TICKER
Global lastTick = MilliSecs()

;;; CREATE INITIAL FORM
CreateForm()

;;; MAIN LOOP
While Not KeyHit(1)
	Cls()
	WaitTimer frameTimer
	
	DrawWalls()
	DrawBricks()
	MoveAndRotateForm()
	Gravity()
	
	Flip 0
Wend
End

;;; FUNCTIONS
Function DrawWalls()
	Local x, y
	For y=0 To 14
		;left wall
		DrawImage wall, 0, y*RESIZE_FACTOR
		
		;right wall
		DrawImage wall, 11*RESIZE_FACTOR, y*RESIZE_FACTOR
	Next
	For x=1 To 10
		;bottom wall
		DrawImage wall, x*RESIZE_FACTOR, 14*RESIZE_FACTOR
	Next
End Function

Function DrawBricks()
	Local b.Brick
	For b.Brick = Each Brick
		DrawImage b\icon, b\x*RESIZE_FACTOR, 14*RESIZE_FACTOR - b\y*RESIZE_FACTOR
	Next
End Function

Function CreateForm()
	;reset fall speed and form rotation
	fallSpeedDelta = FALL_SPEED_INITIAL
	formRotation = 0
	
	Local iconIndex = Rnd(0,BRICK_AMOUNT)
	For i=0 To 3
		form(i) = New Brick : form(i)\icon = brickIcons(iconIndex)
	Next
	
	formSort = Rnd(1,7)
	Select formSort
		;
		; oooo
		Case 1
			form(0)\x = 4 : form(0)\y = BRICK_START_Y-1
			form(1)\x = 5 : form(1)\y = BRICK_START_Y-1
			form(2)\x = 6 : form(2)\y = BRICK_START_Y-1
			form(3)\x = 7 : form(3)\y = BRICK_START_Y-1
		; oo
		; oo
		Case 2
			form(0)\x = 5 : form(0)\y = BRICK_START_Y
			form(1)\x = 6 : form(1)\y = BRICK_START_Y
			form(2)\x = 5 : form(2)\y = BRICK_START_Y-1
			form(3)\x = 6 : form(3)\y = BRICK_START_Y-1
		; o
		; ooo
		Case 3
			form(0)\x = 4 : form(0)\y = BRICK_START_Y
			form(1)\x = 4 : form(1)\y = BRICK_START_Y-1
			form(2)\x = 5 : form(2)\y = BRICK_START_Y-1
			form(3)\x = 6 : form(3)\y = BRICK_START_Y-1
		;   o
		; ooo
		Case 4
			form(0)\x = 4 : form(0)\y = BRICK_START_Y-1
			form(1)\x = 5 : form(1)\y = BRICK_START_Y-1
			form(2)\x = 6 : form(2)\y = BRICK_START_Y-1
			form(3)\x = 6 : form(3)\y = BRICK_START_Y
		; oo
		;  oo
		Case 5
			form(0)\x = 4 : form(0)\y = BRICK_START_Y
			form(1)\x = 5 : form(1)\y = BRICK_START_Y
			form(2)\x = 5 : form(2)\y = BRICK_START_Y-1
			form(3)\x = 6 : form(3)\y = BRICK_START_Y-1
		;  oo
		; oo 
		Case 6
			form(0)\x = 4 : form(0)\y = BRICK_START_Y-1
			form(1)\x = 5 : form(1)\y = BRICK_START_Y-1
			form(2)\x = 5 : form(2)\y = BRICK_START_Y
			form(3)\x = 6 : form(3)\y = BRICK_START_Y
		;  o 
		; ooo
		Case 7
			form(0)\x = 4 : form(0)\y = BRICK_START_Y-1
			form(1)\x = 5 : form(1)\y = BRICK_START_Y-1
			form(2)\x = 5 : form(2)\y = BRICK_START_Y
			form(3)\x = 6 : form(3)\y = BRICK_START_Y-1
		Default
			;for some strange reason, End must be explicitly called to avoid a MAV after RuntimeError...
			RuntimeError "Error: Tried to create invalid form (must be in range 1-7)!" : End
	End Select
End Function

Function MoveAndRotateForm()
	;left and right motion
	If KeyHit(203) Or KeyHit(30) Then
		If form(0)\x > 1 And form(1)\x > 1 And form(2)\x > 1 And form(3)\x > 1 Then
			form(0)\x=form(0)\x-1 : form(1)\x=form(1)\x-1 : form(2)\x=form(2)\x-1 : form(3)\x=form(3)\x-1
		EndIf
	EndIf
	If KeyHit(205) Or KeyHit(32) Then
		If form(0)\x < 10 And form(1)\x < 10 And form(2)\x < 10 And form(3)\x < 10 Then
			form(0)\x=form(0)\x+1 : form(1)\x=form(1)\x+1 : form(2)\x=form(2)\x+1 : form(3)\x=form(3)\x+1
		EndIf
	EndIf
	
	;left and right rotation with required new block fields
	Local rotAngle, newX1, newY1, newX2, newY2, newX3, newY3, newX4, newY4
	
	Local rotLeft = KeyHit(208) Or KeyHit(31)
	Local rotRight = KeyHit(200) Or KeyHit(17)
	
	If rotLeft Or rotRight Then
		If rotLeft Then
			rotAngle = -90
		Else
			rotAngle = 90
		EndIf
		
		Select formSort
			Case 1
				If formRotation = 0 Or formRotation = 180 Then
					newX1 = form(0)\x+1 : newY1 = form(0)\y-1
					newX2 = form(1)\x : newY2 = form(1)\y
					newX3 = form(2)\x-1 : newY3 = form(2)\y+1
					newX4 = form(3)\x-2 : newY4 = form(3)\y+2
				ElseIf formRotation = 90 Or formRotation = 270 Then
					newX1 = form(0)\x-1 : newY1 = form(0)\y+1
					newX2 = form(1)\x : newY2 = form(1)\y
					newX3 = form(2)\x+1 : newY3 = form(2)\y-1
					newX4 = form(3)\x+2 : newY4 = form(3)\y-2
				EndIf
			Case 3
				If rotRight Then
					If formRotation = 0 Then
						newX1 = form(0)\x+2 : newY1 = form(0)\y
						newX2 = form(1)\x+1 : newY2 = form(1)\y-1
						newX3 = form(2)\x : newY3 = form(2)\y
						newX4 = form(3)\x-1 : newY4 = form(3)\y+1
					ElseIf formRotation = 90 Then
						newX1 = form(0)\x : newY1 = form(0)\y-2
						newX2 = form(1)\x+1 : newY2 = form(1)\y+1
						newX3 = form(2)\x : newY3 = form(2)\y
						newX4 = form(3)\x-1 : newY4 = form(3)\y-1
					ElseIf formRotation = 180 Then
						newX1 = form(0)\x-2 : newY1 = form(0)\y
						newX2 = form(1)\x-1 : newY2 = form(1)\y+1
						newX3 = form(2)\x : newY3 = form(2)\y
						newX4 = form(3)\x+1 : newY4 = form(3)\y-1
					ElseIf formRotation = 270 Then
						newX1 = form(0)\x : newY1 = form(0)\y+2
						newX2 = form(1)\x-1 : newY2 = form(1)\y-1
						newX3 = form(2)\x : newY3 = form(2)\y
						newX4 = form(3)\x+1 : newY4 = form(3)\y+1
					EndIf
				Else If rotLeft Then
					If formRotation = 90 Then
						newX1 = form(0)\x-2 : newY1 = form(0)\y
						newX2 = form(1)\x-1 : newY2 = form(1)\y+1
						newX3 = form(2)\x : newY3 = form(2)\y
						newX4 = form(3)\x+1 : newY4 = form(3)\y-1
					ElseIf formRotation = 180 Then
						newX1 = form(0)\x : newY1 = form(0)\y+2
						newX2 = form(1)\x-1 : newY2 = form(1)\y-1
						newX3 = form(2)\x : newY3 = form(2)\y
						newX4 = form(3)\x+1 : newY4 = form(3)\y+1
					ElseIf formRotation = 270 Then
						newX1 = form(0)\x+2 : newY1 = form(0)\y
						newX2 = form(1)\x+1 : newY2 = form(1)\y-1
						newX3 = form(2)\x : newY3 = form(2)\y
						newX4 = form(3)\x-1 : newY4 = form(3)\y+1
					ElseIf formRotation = 0 Then
						newX1 = form(0)\x : newY1 = form(0)\y-2
						newX2 = form(1)\x+1 : newY2 = form(1)\y+1
						newX3 = form(2)\x : newY3 = form(2)\y
						newX4 = form(3)\x-1 : newY4 = form(3)\y-1
					EndIf
				EndIf
			Case 4
				If rotRight Then
					If formRotation = 0 Then
						newX1 = form(0)\x+1 : newY1 = form(0)\y+1
						newX2 = form(1)\x : newY2 = form(1)\y
						newX3 = form(2)\x-1 : newY3 = form(2)\y-1
						newX4 = form(3)\x : newY4 = form(3)\y-2
					ElseIf formRotation = 90 Then
						newX1 = form(0)\x+1 : newY1 = form(0)\y-1
						newX2 = form(1)\x : newY2 = form(1)\y
						newX3 = form(2)\x-1 : newY3 = form(2)\y+1
						newX4 = form(3)\x-2 : newY4 = form(3)\y
					ElseIf formRotation = 180 Then
						newX1 = form(0)\x-1 : newY1 = form(0)\y-1
						newX2 = form(1)\x : newY2 = form(1)\y
						newX3 = form(2)\x+1 : newY3 = form(2)\y+1
						newX4 = form(3)\x : newY4 = form(3)\y+2
					ElseIf formRotation = 270 Then
						newX1 = form(0)\x-1 : newY1 = form(0)\y+1
						newX2 = form(1)\x : newY2 = form(1)\y
						newX3 = form(2)\x+1 : newY3 = form(2)\y-1
						newX4 = form(3)\x+2 : newY4 = form(3)\y
					EndIf
				Else If rotLeft Then
					If formRotation = 90 Then
						newX1 = form(0)\x-1 : newY1 = form(0)\y-1
						newX2 = form(1)\x : newY2 = form(1)\y
						newX3 = form(2)\x+1 : newY3 = form(2)\y+1
						newX4 = form(3)\x : newY4 = form(3)\y+2
					ElseIf formRotation = 180 Then
						newX1 = form(0)\x-1 : newY1 = form(0)\y+1
						newX2 = form(1)\x : newY2 = form(1)\y
						newX3 = form(2)\x+1 : newY3 = form(2)\y-1
						newX4 = form(3)\x+2 : newY4 = form(3)\y
					ElseIf formRotation = 270 Then
						newX1 = form(0)\x+1 : newY1 = form(0)\y+1
						newX2 = form(1)\x : newY2 = form(1)\y
						newX3 = form(2)\x-1 : newY3 = form(2)\y-1
						newX4 = form(3)\x : newY4 = form(3)\y-2
					ElseIf formRotation = 0 Then
						newX1 = form(0)\x+1 : newY1 = form(0)\y-1
						newX2 = form(1)\x : newY2 = form(1)\y
						newX3 = form(2)\x-1 : newY3 = form(2)\y+1
						newX4 = form(3)\x-2 : newY4 = form(3)\y
					EndIf
				EndIf				
			Case 5
				If formRotation = 0 Or formRotation = 180 Then
					newX1 = form(0)\x+1 : newY1 = form(0)\y+1
					newX2 = form(1)\x : newY2 = form(1)\y
					newX3 = form(2)\x-1 : newY3 = form(2)\y+1
					newX4 = form(3)\x-2 : newY4 = form(3)\y
				ElseIf formRotation = 90 Or formRotation = 270 Then
					newX1 = form(0)\x-1 : newY1 = form(0)\y-1
					newX2 = form(1)\x : newY2 = form(1)\y
					newX3 = form(2)\x+1 : newY3 = form(2)\y-1
					newX4 = form(3)\x+2 : newY4 = form(3)\y
				EndIf
			Case 6
				If formRotation = 0 Or formRotation = 180 Then
					newX1 = form(0)\x+1 : newY1 = form(0)\y+1
					newX2 = form(1)\x : newY2 = form(1)\y
					newX3 = form(2)\x+1 : newY3 = form(2)\y-1
					newX4 = form(3)\x : newY4 = form(3)\y-2
				ElseIf formRotation = 90 Or formRotation = 270 Then
					newX1 = form(0)\x-1 : newY1 = form(0)\y-1
					newX2 = form(1)\x : newY2 = form(1)\y
					newX3 = form(2)\x-1 : newY3 = form(2)\y+1
					newX4 = form(3)\x : newY4 = form(3)\y+2
				EndIf
			Case 7
				If rotRight Then
					If formRotation = 0 Then
						newX1 = form(0)\x+1 : newY1 = form(0)\y+1
						newX2 = form(1)\x : newY2 = form(1)\y
						newX3 = form(2)\x+1 : newY3 = form(2)\y-1
						newX4 = form(3)\x-1 : newY4 = form(3)\y-1
					ElseIf formRotation = 90 Then
						newX1 = form(0)\x+1 : newY1 = form(0)\y-1
						newX2 = form(1)\x : newY2 = form(1)\y
						newX3 = form(2)\x-1 : newY3 = form(2)\y-1
						newX4 = form(3)\x-1 : newY4 = form(3)\y+1
					ElseIf formRotation = 180 Then
						newX1 = form(0)\x-1 : newY1 = form(0)\y-1
						newX2 = form(1)\x : newY2 = form(1)\y
						newX3 = form(2)\x-1 : newY3 = form(2)\y+1
						newX4 = form(3)\x+1 : newY4 = form(3)\y+1
					ElseIf formRotation = 270 Then
						newX1 = form(0)\x-1 : newY1 = form(0)\y+1
						newX2 = form(1)\x : newY2 = form(1)\y
						newX3 = form(2)\x+1 : newY3 = form(2)\y+1
						newX4 = form(3)\x+1 : newY4 = form(3)\y-1
					EndIf
				ElseIf rotLeft Then
					If formRotation = 90 Then
						newX1 = form(0)\x-1 : newY1 = form(0)\y-1
						newX2 = form(1)\x : newY2 = form(1)\y
						newX3 = form(2)\x-1 : newY3 = form(2)\y+1
						newX4 = form(3)\x+1 : newY4 = form(3)\y+1
					ElseIf formRotation = 180 Then
						newX1 = form(0)\x-1 : newY1 = form(0)\y+1
						newX2 = form(1)\x : newY2 = form(1)\y
						newX3 = form(2)\x+1 : newY3 = form(2)\y+1
						newX4 = form(3)\x+1 : newY4 = form(3)\y-1
					ElseIf formRotation = 270 Then
						newX1 = form(0)\x+1 : newY1 = form(0)\y+1
						newX2 = form(1)\x : newY2 = form(1)\y
						newX3 = form(2)\x+1 : newY3 = form(2)\y-1
						newX4 = form(3)\x-1 : newY4 = form(3)\y-1
					ElseIf formRotation = 0 Then
						newX1 = form(0)\x+1 : newY1 = form(0)\y-1
						newX2 = form(1)\x : newY2 = form(1)\y
						newX3 = form(2)\x-1 : newY3 = form(2)\y-1
						newX4 = form(3)\x-1 : newY4 = form(3)\y+1
					EndIf
				EndIf
		End Select
	EndIf
	
	If rotAngle <> 0 And block(newX1,newY1)=0 And block(newX2,newY2)=0 And block(newX3,newY3)=0 And block(newX4,newY4)=0 Then
		form(0)\x = newX1 : form(0)\y = newY1
		form(1)\x = newX2 : form(1)\y = newY2
		form(2)\x = newX3 : form(2)\y = newY3
		form(3)\x = newX4 : form(3)\y = newY4
		formRotation = formRotation + rotAngle
		If formRotation >= 360 Then formRotation = 360 - formRotation
		If formRotation < 0 Then formRotation = 360 + formRotation
	EndIf
	
	If KeyHit(57) Then fallSpeedDelta = 50
End Function

Function Gravity()
	If form(0) = Null Or form(1) = Null Or form(2) = Null Or form(3) = Null Then Return
	
	Local ms = MilliSecs()
	If ms > lastTick + fallSpeedDelta Then
		lastTick = ms
		
		Local i, j
		For i=0 To 3
			;check for ground or brick collision
			If form(i)\y = 1 Or block(form(i)\x,form(i)\y-1) Then
				For j=0 To 3
					block(form(j)\x,form(j)\y) = 1
				Next
				
				If form(i)\y >= BRICK_START_Y-1 Then
					;for some strange reason, End must be explicitly called to avoid a MAV after RuntimeError...
					RuntimeError "GAME OVER" : End
				Else
					CheckRowDeletion()
					CreateForm()
					Return
				EndIf
			EndIf
		Next
		
		For i=0 To 3
			form(i)\y = form(i)\y - 1
		Next
	EndIf
End Function

Function CheckRowDeletion()
	Local x, y, r.Row, b.Brick
	
	;check all rows for deletion
	For y=14 To 1 Step -1
		Local bricksInRow = 0
		
		For x=1 To 10
			If block(x,y) = 1 Then bricksInRow = bricksInRow+1
		Next
		
		If bricksInRow = 10 Then
			r.Row = New Row
			r\y = y
		EndIf
		
		If bricksInRow > 0 Then fallSpeedDelta = fallSpeedDelta - 50
	Next
	
	;delete all filled rows
	For r.Row = Each Row
		
		;delete actual row
		For x=1 To 10
			If block(x,r\y) = 1 Then
				block(x,r\y) = 0
				
				For b.Brick = Each Brick
					If b\x = x And b\y = r\y Then
						Delete b
						Exit
					EndIf
				Next
			EndIf
		Next
		
		;move upper rows down successively
		For y=r\y+1 To 14
			For x=1 To 10
				If block(x,y) = 1 Then
					block(x,y) = 0
					block(x,y-1) = 1
					DebugLog "moved "+x+", "+y+" to "+x+", "+(y-1)+": "+block(x,y-1)
					
					For b.Brick = Each Brick
						If b\x = x And b\y = y Then
							b\y = b\y-1
							Exit
						EndIf
					Next
				EndIf
			Next
		Next
		
		Delete r
	Next
End Function


;;; TYPES
Type Brick
	Field x, y, icon
End Type

Type Row
	Field y
End Type
;~IDEal Editor Parameters:
;~C#Blitz3D